@@ -21,7 +21,7 @@ link_files = {
                 url='{package_url}/issues/{issue}',
             ),
             dict(
-                pattern=r'^(?m)((?P<scm_version>v?\d+(\.\d+){1,2}))\n[-=]+\n',
+                pattern=r'(?m:^((?P<scm_version>v?\d+(\.\d+){1,2}))\n[-=]+\n)',
                 with_scm='{text}\n{rev[timestamp]:%d %b %Y}\n',
             ),
             dict(
@@ -32,6 +32,9 @@ link_files = {
     )
 }
 
+# Be strict about any broken references:
+nitpicky = True
+
 # Custom sidebar templates, maps document names to template names.
 html_theme = 'alabaster'
 templates_path = ['_templates']
